#include <iostream>

extern "C" {
#include "rspl/rspl.h"
}

using namespace std;

#include "1dinterp.h"


// Base class for 1D interpolation using the rspl library from Argyll.

class Interpolate_1D_Private
{
	public:
	Interpolate_1D_Private() : interp(NULL)
	{
		interp=new_rspl(0,1,1); // Create a new 1d -> 1d RSPL structure
		gres[0]=255;
		avgdev[0]=0.1;
	}
	~Interpolate_1D_Private()
	{
		if(interp)
			interp->del(interp);
	}
	bool Populate(int count, double *in, double *out,double smoothing)
	{
		cow *myco=new cow[count];
		datai ilow,ihigh;
		datao olow,ohigh;

		ilow[0]=ihigh[0]=in[0];
		olow[0]=ohigh[0]=out[0];

		for(int i=0;i<count;++i)
		{
			myco[i].p[0]=in[i];
			myco[i].v[0]=out[i];
			myco[i].w=1.0;
			if(in[i]<ilow[0]) ilow[0]=in[i];
			if(in[i]>ihigh[0]) ihigh[0]=in[i];
			if(out[i]<olow[0]) olow[0]=out[i];
			if(out[i]>ohigh[0]) ohigh[0]=out[i];
		}
		// Increase weights of endpoints...
		myco[0].w=10.0;
		myco[count-1].w=10.0;

		// Add some "headroom"
		ilow[0]-=1.0;
		ihigh[0]+=1.0;
		olow[0]-=1.0;
		ohigh[0]+=1.0;

		interp->fit_rspl_w(interp,
		           RSPL_EXTRAFIT,				/* Non-mon and clip flags */
		           myco,			/* Test points */
		           count,				/* Number of test points */
		           ilow, ihigh, gres,		/* Low, high, resolution of grid */
		           olow, ohigh,			/* Default data scale */
		           smoothing,			/* Smoothing */
		           avgdev,				/* Average deviation */
		           NULL);				/* iwidth */
		delete[] myco;
		return(true);
	}
	protected:
	rspl *interp;
	int gres[MXDI];
	double avgdev[MXDO];
	friend class Interpolate_1D;
};


Interpolate_1D::Interpolate_1D() : priv(NULL)
{
	priv=new Interpolate_1D_Private();
}


Interpolate_1D::~Interpolate_1D()
{
	if(priv)
		delete priv;
}


double Interpolate_1D::Interpolate(double input)
{
	co point;
	point.p[0]=input;
	point.v[0]=0.0;
	if(priv->interp->interp(priv->interp,&point))
		cerr << "Grid clipped... ";
	return(point.v[0]);
}


double Interpolate_1D::ReverseInterpolate(double output)
{
	co point;
	point.v[0]=output;
	point.p[0]=0.0;
	int result=priv->interp->rev_interp(priv->interp,0,1,NULL,NULL,&point);
	if(result==0)
	{
		cerr << "No solutions found for " << output << endl;
		return(0.0);
	}
//	cerr << "Mapping " << output << " to " << point.p[0] << endl;
	return(point.p[0]);
}


bool Interpolate_1D::Populate(int count,double *in, double *out,double smoothing)
{
	return(priv->Populate(count,in,out,smoothing));
}

