#include <iostream>
#include <math.h>

#include <gtk/gtk.h>
#include <cairo.h>

#include "imagesource_devicen_preview.h"

#include "channelresponse.h"

using namespace std;


static void paint (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, width/4,0);
	cairo_line_to(cr,width/4,height);
	cairo_move_to (cr, width/2,0);
	cairo_line_to(cr,width/2,height);
	cairo_move_to (cr, 0, height/2);
	cairo_line_to(cr,width/2,height/2);
	cairo_move_to (cr, width/2, height-1);
	cairo_line_to(cr,width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, width/4+5, 12);
	cairo_show_text (cr, "+128");
	cairo_move_to (cr, width/4+5, height-3);
	cairo_show_text (cr, "-128");

	cairo_move_to (cr, 5, height/2+15);
	cairo_show_text (cr, "-128");
	cairo_move_to (cr, width/2-30, height/2+15);
	cairo_show_text (cr, "+128");

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width/4+50, 20);
	cairo_show_text (cr, "b");
	cairo_move_to (cr, width/2-20, height/2+35);
	cairo_show_text (cr, "a");
	cairo_move_to (cr, width/2+15, 12);
	cairo_show_text (cr, "L*");
	cairo_move_to (cr, width-20, height-5);
	cairo_show_text (cr, "c");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		for(int i=0;i<sc;++i)
		{
			LabValue l(sam[i]);
			cairo_move_to(cr,width/4+(l.a*width)/512-1,height/2-(l.b*height)/256-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			float c=sqrt(l.a*l.a+l.b*l.b);
			cairo_move_to(cr,width/2+(c*width)/360-1,height/2-(l.L*height)/200-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_Lstar (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "L* 100");
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();

		float lmin=100.0;
		float lmax=0.0;
		for(int i=0;i<sc;++i)
		{
			LabValue l(sam[i]);
			if(l.L<lmin)
				lmin=l.L;
			if(l.L>lmax)
				lmax=l.L;
		}

		cairo_move_to(cr,0,height-(paperlab.L*height)/100);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			
			cairo_line_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height-(l.L*height)/100);
		}

		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_Lstar_linearized (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "L* 100");
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();

		float lmin=100.0;
		float lmax=0.0;
		for(int i=0;i<sc;++i)
		{
			LabValue l(sam[i]);
			if(l.L<lmin)
				lmin=l.L;
			if(l.L>lmax)
				lmax=l.L;
		}

		cairo_move_to(cr,0,height-(paperlab.L*height)/100);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			input=c->Interpolate(input);
			
			cairo_move_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
		}

		cairo_move_to(cr,0,height-(height*lmax)/100.0);
		cairo_line_to(cr,width-1,height-(height*lmin)/100.0);

		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_a (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "a 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "a -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.a*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_b (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "b 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "b -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.b*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


#define DEFAULT_WIDTH 640
#define DEFAULT_HEIGHT 480

int main(int argc, char *argv[])
{
	gtk_init(&argc, &argv);
	if(argc>1)
	{
		try
		{
			char *filename=argv[1];
			RawChannelResponses ti3(filename);

			GtkWidget *window;
			GtkWidget *canvas;

			window   = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			g_signal_connect(G_OBJECT (window), "delete-event",
			                G_CALLBACK (gtk_main_quit), NULL);

			GtkWidget *vbox=gtk_vbox_new(TRUE,0);
			gtk_container_add(GTK_CONTAINER (window), vbox);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_Lstar), &ti3);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);

			GtkWidget *hbox=gtk_hbox_new(TRUE,0);
			gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_Lstar_linearized), &ti3);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);

#if 0
			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_b), &ti3);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);
#endif
			gtk_widget_show_all(window);

			gtk_main ();
		}
		catch(const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	return(0);
}


