#include <iostream>

#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_gamma.h"

#include "linearization.h"

using namespace std;


// ImageSource for applying curves

class ImageSource_CorrectionCurves : public ImageSource
{
	public:
	ImageSource_CorrectionCurves(ImageSource *source,CorrectionCurve *curves)
		: ImageSource(source), source(source), curves(curves)
	{
		MakeRowBuffer();
	}

	~ImageSource_CorrectionCurves()
	{
		if(source)
			delete source;
	}

	ISDataType *GetRow(int row)
	{
		ISDataType *src;

		if(row==currentrow)
			return(rowbuffer);

		src=source->GetRow(row);

		if(HAS_ALPHA(type))
		{
			for(int i=0;i<width;++i)
			{
				int j;
				for(j=0;j<samplesperpixel-1;++j)
					rowbuffer[i*samplesperpixel+j]=curves[j][src[i*samplesperpixel+j]];
				rowbuffer[i*samplesperpixel+j]=src[i*samplesperpixel+j];	// Copy alpha unchanged
			}	
		}
		else
		{
			for(int i=0;i<width;++i)
			{
				int j;
				for(j=0;j<samplesperpixel;++j)
					rowbuffer[i*samplesperpixel+j]=curves[j][src[i*samplesperpixel+j]];
			}	
		}
		currentrow=row;
		return(rowbuffer);
	}
	protected:
	ImageSource *source;
	CorrectionCurve *curves;
};


// Linearization class

ConfigTemplate Linearization::Template[]=
{
	ConfigTemplate("BaseDensity",float(1.0)),
	ConfigTemplate("BaseGamma",float(1.0)),
	ConfigTemplate()
};


Linearization::Linearization(ConfigFile *file) : ConfigDB(Template)
{
	new ConfigDBHandler(file,"[Linearization]",this);
}


Linearization::~Linearization()
{

}


ImageSource *Linearization::Apply(ImageSource *source)
{
	// Apply curves here...
	source=new ImageSource_CorrectionCurves(source,curves);

	source=new ImageSource_ModifiedGamma(source,FindFloat("BaseGamma"));
//	source=new ImageSource_Gamma(source,FindFloat("BaseGamma"));

	ISDeviceNValue density(source->samplesperpixel,ISDataType(FindFloat("BaseDensity")*IS_SAMPLEMAX));
	source=new ImageSource_ScaleDensity(source,density);

	return(source);
}


CorrectionCurve &Linearization::operator[](int i)
{
	if(i<0 || i>=LINEARIZATION_MAX_CHANNELS)
		throw "Linearization - Curve index out of range!";
	return(curves[i]);
}

