#include <iostream>

#include "linearizedprinter.h"
#include "correctioncurve.h"
#include "channelresponse.h"
#include "gpdevicensupport.h"

using namespace std;

// Subclass of CorrectionCurve - used within SetCurvesFromTI3()

class CorrectionCurve_ChannelResponse : public CorrectionCurve
{
	public:
	CorrectionCurve_ChannelResponse(ChannelResponse *c) : CorrectionCurve()
	{
		for(int i=0;i<=IS_SAMPLEMAX;++i)
		{
			double in=(i*100.0)/IS_SAMPLEMAX;
			double out=c->ReverseInterpolate(in);
			if(out<0.0) out=0.0;
			if(out>100.0) out=100.0;
			ISDataType o=ISDataType((IS_SAMPLEMAX*out)/100.0);
			correction[i]=o;
//			cerr << "Mapping " << i << " to " << o << endl;
		}
	}
	protected:
};



LinearizedPrinter::LinearizedPrinter(PrintOutput &output,ConfigFile *inf,const char *section)
	: GPrinter(output,inf,section), linearization(inf)
{
}


LinearizedPrinter::~LinearizedPrinter()
{
}


void LinearizedPrinter::Print(ImageSource *source,int xpos,int ypos)
{
	source=linearization.Apply(source);
	GPrinter::Print(source,xpos,ypos);
}


void LinearizedPrinter::SetCurvesFromTI3(const char *fn)
{
	GPColorantList colorants(this);

	RawChannelResponses resp(fn);

	ChannelResponse *c=resp.FirstChannel();
	while(c)
	{
		cerr << "Assigning channel " << c->GetName() << endl;
		cerr << "Channel index is: " << colorants.GetColorantIndex(c->GetLongName()) << endl;
		CorrectionCurve_ChannelResponse temp(c);
		linearization[colorants.GetColorantIndex(c->GetLongName())]=temp;
		c=c->NextChannel();
	}
}

