#include <iostream>

#include <gtk/gtk.h>

#include "miscwidgets/generaldialogs.h"
#include "miscwidgets/pixbufview.h"
#include "miscwidgets/imageselector.h"
#include "support/util.h"
#include "support/progressbar.h"

#include "stpui_widgets/stpui_combo.h"
#include "stpui_widgets/stpui_printerselector.h"
#include "stpui_widgets/stpui_optionbook.h"
#include "gp_cppsupport/printoutputselector.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_devicen_remap.h"

#include "stepwidget_printcharts.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


// Chart printing "step"

StepWidget_PrintCharts::StepWidget_PrintCharts(ProjectGUI *gui) : StepWidget(gui), colorants(&gui->gprinter), ti1(NULL), tiffs(NULL)
{
	try
	{
		ti1=gui->GetRawChannelLinChart("RawLin");
		char *fn=gui->GetTempRoot("RawLin");
		if(fn)
		{
			tiffs=new Argyll_TestChart_TIFFs(fn);
			free(fn);
		}
	}
	catch(const char *err)
	{
		cerr << "Error: " << err << endl;
	}

	widget=gtk_vbox_new(FALSE,0);

	GtkWidget *hbox=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(widget),GTK_WIDGET(hbox),TRUE,TRUE,0);
	gtk_widget_show(hbox);

	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(vbox),TRUE,TRUE,0);
	gtk_widget_show(vbox);

	// PixbufView

	pview=pixbufview_new(NULL,false);
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(pview),TRUE,TRUE,0);
	gtk_widget_show(pview);


	// Print Button

	GtkWidget *tmp=gtk_button_new_with_label(_("Print test chart"));
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(tmp),FALSE,FALSE,0);
	g_signal_connect(G_OBJECT(tmp),"clicked",G_CALLBACK(printclicked),this);
	gtk_widget_show(tmp);


	// ImageSelector

	imageselector=imageselector_new(NULL,true,false);
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(imageselector),FALSE,FALSE,8);
	gtk_widget_show(imageselector);

	if(tiffs)
	{
		char *fn=tiffs->GetFirstFilename();
		while(fn)
		{
			char *jpegfn=BuildFilename(fn,"","jpg");
			if(g_path_is_absolute(jpegfn))
			{
				imageselector_set_filename(IMAGESELECTOR(imageselector),jpegfn);
			}
			else
			{
				char *absfn=BuildAbsoluteFilename(jpegfn);
				imageselector_set_filename(IMAGESELECTOR(imageselector),absfn);
				free(absfn);
			}
			free(jpegfn);
			free(fn);
			fn=tiffs->GetNextFilename();
		}
	}

	g_signal_connect(G_OBJECT(imageselector),"changed",G_CALLBACK(imagechanged),this);

	DisplayChart();
}


StepWidget_PrintCharts::~StepWidget_PrintCharts()
{
	if(tiffs)
		delete tiffs;
	if(ti1)
		delete ti1;
}


const char *StepWidget_PrintCharts::GetDirections()
{
	return(_("Test-chart printing:  Please use the image selector on the right to "
			"select a page.  You can examine the page in the preview below. "
			"Use the Print button to print each page in turn."));
}


void StepWidget_PrintCharts::DisplayChart()
{
	const char *jfn=imageselector_get_filename(IMAGESELECTOR(imageselector));
	if(jfn)
	{
		char *fn=BuildFilename(jfn,"","tif");
		if(fn)
		{
			GdkPixbuf *pb=gui->GetTestChartPreview(ti1,fn);
			if(pb)
			{
				pixbufview_set_pixbuf(PIXBUFVIEW(pview),pb);
				g_object_unref(G_OBJECT(pb));
			}
			free(fn);
		}
	}
}


void StepWidget_PrintCharts::PrintChart()
{
	const char *jfn=imageselector_get_filename(IMAGESELECTOR(imageselector));
	if(jfn)
	{
		char *fn=BuildFilename(jfn,"","tif");
		if(fn)
		{
			try
			{
				int colcount=colorants.GetColorantCount();
				int *channelmap=gui->CreateRawChannelMap(colorants,*ti1);

				ImageSource *is=ISLoadImage(fn);

				is=new ImageSource_ToDeviceN(is);
				if(channelmap)
				{
					is=new ImageSource_DeviceN_Remap(is,channelmap,colcount);
					cerr << "Have a channel map" << endl;
				}
				else
					cerr << "No channel map" << endl;
				gui->Print(is);
				delete is;
				delete[] channelmap;
			}
			catch (const char *err)
			{
				ErrorMessage_Dialog(err,gui->window);
			}
			free(fn);
		}
	}
}


void StepWidget_PrintCharts::printclicked(GtkWidget *wid,gpointer data)
{
	StepWidget_PrintCharts *step=(StepWidget_PrintCharts *)data;
	try
	{
		step->PrintChart();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,step->gui->window);
	}
}


void StepWidget_PrintCharts::imagechanged(GtkWidget *wid,gpointer data)
{
	StepWidget_PrintCharts *step=(StepWidget_PrintCharts *)data;
	try
	{
		step->DisplayChart();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,step->gui->window);
	}
}

