#ifndef TI3FILE_H
#define TI3FILE_H

#include "cgats/cgats.h"
#include "imagesource/devicencolorant.h"


// Simplistic wrapper around opening a CGATS file and finding indices for various
// variables within.

class CGATSFile
{
	public:
	CGATSFile(const char *filename,const char *type=NULL);
	virtual ~CGATSFile();
	virtual const char *GetFieldName(int table,int idx);
	virtual int FindField(int table,const char *name);
	virtual const char *GetKeyword(int table,const char *key);
	protected:
	cgats *cgatsfile;
};


// TI1 file wrapper - doesn't provide access to the actual data, but
// builds a list of colorants, accessible via the DeviceNColorantList
// interface, for use in interpreting the DeviceN TIFFs produced by printtarg

class TI1File : public CGATSFile, public DeviceNColorantList
{
	public:
	TI1File(const char *filename);
	virtual ~TI1File();
	virtual int GetColorantIndex(const char *name);
	protected:
	const char *colorantnames;
};


// TI3 file wrapper - provides access to the data from a successfully
// read chart.
// Abstracted yet further for linearization purposes by the
// RawChannelResponses class

class TI3File : public CGATSFile
{
	public:
	TI3File(const char *filename);
	virtual ~TI3File();
	virtual int GetColorantCount();
	virtual int GetSets();
	virtual double **GetData(int set);
	virtual const char *GetFieldName(int idx);
	virtual int FindField(const char *name);
	protected:
	cgats_table *table;
	int colorants;
	int sets;
	int Xindex,Yindex,Zindex;
};

#endif

